@@ -43,8 +43,8 @@ namespace eval portpkg {
 }
 
 # define options
-options package.type package.destpath package.flat package.resources package.scripts
-options pkg.asroot
+options package.type package.destpath package.flat package.resources \
+        package.scripts pkg.asroot
 
 # Set defaults
 default package.destpath {${workpath}}
@@ -57,11 +57,13 @@ default pkg.asroot no
 set_ui_prefix
 
 proc portpkg::pkg_start {args} {
-    global packagemaker_path portpkg::packagemaker portpkg::pkgbuild \
-           portpkg::language xcodeversion portpath porturl \
+    global packagemaker_path xcodeversion porturl \
            package.resources package.scripts package.flat \
            subport version revision description long_description \
            homepage workpath os.major
+    variable packagemaker
+    variable pkgbuild
+    variable language
 
     if {[catch {findBinary pkgbuild /usr/bin/pkgbuild} pkgbuild]} {
         set pkgbuild ""
@@ -75,7 +77,8 @@ proc portpkg::pkg_start {args} {
                     ui_warn "PackageMaker.app not found; you may need to install it or set packagemaker_path in macports.conf"
                 }
             } else {
-                set packagemaker_path "[option developer_dir]/Applications/Utilities/PackageMaker.app"
+                global developer_dir
+                set packagemaker_path ${developer_dir}/Applications/Utilities/PackageMaker.app
             }
         }
         set packagemaker "${packagemaker_path}/Contents/MacOS/PackageMaker"
@@ -104,7 +107,7 @@ proc portpkg::pkg_start {args} {
 }
 
 proc portpkg::pkg_main {args} {
-    global subport epoch version revision UI_PREFIX
+    global subport epoch version revision
 
     if {[getuid] == 0 && [geteuid] != 0} {
         elevateToRoot "pkg"
@@ -114,10 +117,13 @@ proc portpkg::pkg_main {args} {
 }
 
 proc portpkg::package_pkg {portname portepoch portversion portrevision} {
-    global UI_PREFIX portdbpath destpath workpath prefix description \
-    package.flat package.destpath portpath os.version os.major \
-    package.resources package.scripts portpkg::packagemaker \
-    pkg_post_unarchive_deletions portpkg::language portpkg::pkgbuild
+    global UI_PREFIX destpath workpath prefix description \
+        package.flat package.destpath portpath os.major \
+        package.resources package.scripts \
+        pkg_post_unarchive_deletions
+    variable packagemaker
+    variable language
+    variable pkgbuild
 
     set pkgpath "${package.destpath}/[image_name $portname $portversion $portrevision].pkg"
 
@@ -163,7 +169,7 @@ proc portpkg::package_pkg {portname portepoch portversion portrevision} {
             if {$using_pkgbuild} {
                 set cmdline "$pkgbuild --root [shellescape ${destpath}] ${pkgresources} --info [shellescape $infofile] --install-location / --identifier org.macports.$portname"
             } else {
-                set cmdline "PMResourceLocale=${language} $packagemaker --root [shellescape ${destpath}] --out [shellescape ${pkgpath}] ${pkgresources} --info [shellescape $infofile] --target $pkgtarget --domain system --id org.macports.$portname"
+                set cmdline "PMResourceLocale=${language} [shellescape $packagemaker] --root [shellescape ${destpath}] --out [shellescape ${pkgpath}] ${pkgresources} --info [shellescape $infofile] --target $pkgtarget --domain system --id org.macports.$portname"
             }
             if {${os.major} >= 10} {
                 set v [mp_version_to_apple_version $portepoch $portversion $portrevision]
@@ -193,7 +199,7 @@ proc portpkg::package_pkg {portname portepoch portversion portrevision} {
         } else {
             write_info_plist ${workpath}/Info.plist $portname $portversion $portrevision
             write_description_plist ${workpath}/Description.plist $portname $portversion $description
-            system "$packagemaker -build -f [shellescape ${destpath}] -p [shellescape ${pkgpath}] -r [shellescape ${package.resources}] -i [shellescape ${workpath}/Info.plist] -d [shellescape ${workpath}/Description.plist]"
+            system "[shellescape $packagemaker] -build -f [shellescape ${destpath}] -p [shellescape ${pkgpath}] -r [shellescape ${package.resources}] -i [shellescape ${workpath}/Info.plist] -d [shellescape ${workpath}/Description.plist]"
         }
 
         file delete ${workpath}/Info.plist \
@@ -402,6 +408,21 @@ proc portpkg::write_package_info {infofile} {
     close $infofd
 }
 
+# Return a hostArchitectures Distribution attribute reflecting the
+# architectures that this pkg will work on.
+proc portpkg::get_hostArchitectures {} {
+    global supported_archs configure.build_arch configure.universal_archs
+    if {$supported_archs eq "noarch"} {
+        return \ hostArchitectures="arm64,i386,ppc,x86_64"
+    } elseif {[variant_exists universal] && [variant_isset universal] && [llength ${configure.universal_archs}] >= 2} {
+        return \ hostArchitectures="[join [lsort -ascii ${configure.universal_archs}] ,]"
+    } elseif {${configure.build_arch} ne ""} {
+        return \ hostArchitectures="${configure.build_arch}"
+    } else {
+        return {}
+    }
+}
+
 proc portpkg::write_distribution {dfile portname portversion portrevision} {
     global macosx_deployment_target
     set portname_e [xml_escape $portname]
@@ -409,7 +430,7 @@ proc portpkg::write_distribution {dfile portname portversion portrevision} {
     puts $dfd "<?xml version=\"1.0\" encoding=\"utf-8\"?>
 <installer-gui-script minSpecVersion=\"1\">
     <title>${portname_e}</title>
-    <options customize=\"never\"/>
+    <options customize=\"never\"[get_hostArchitectures]/>
     <allowed-os-versions><os-version min=\"${macosx_deployment_target}\"/></allowed-os-versions>
     <background file=\"background.tiff\" mime-type=\"image/tiff\" alignment=\"bottomleft\" scaling=\"none\"/>
     <welcome mime-type=\"text/html\" file=\"Welcome.html\"/>
