@@ -1,7 +1,7 @@
 /*
  * adv-flock.c
  *
- * Copyright (c) 2009 The MacPorts Project
+ * Copyright (c) 2009, 2014-2018 The MacPorts Project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,10 +33,11 @@
 #include <config.h>
 #endif
 
-/* needed to get struct sigaction on some platforms */
+#ifndef __APPLE__
+/* needed to get struct sigaction on some platforms, but
+  hides flock on macOS */
 #define _XOPEN_SOURCE 500L
-/* the above hides flock on OS X without _DARWIN_C_SOURCE */
-#define _DARWIN_C_SOURCE
+#endif
 
 #if HAVE_SYS_FILE_H
 #include <sys/file.h>
@@ -59,12 +60,12 @@ static void alarmHandler(int sig UNUSED) {
 }
 
 int
-AdvFlockCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+AdvFlockCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]) {
     static const char errorstr[] = "use one of \"-shared\", \"-exclusive\", or \"-unlock\", and optionally \"-noblock\"";
     int operation = 0, fd, i, ret, sigret = TCL_OK;
     int errnoval = 0;
     int oshared = 0, oexclusive = 0, ounlock = 0, onoblock = 0, retry = 0;
-#if defined(HAVE_LOCKF) && !defined(HAVE_FLOCK)
+#if !defined(HAVE_FLOCK)
     off_t curpos;
 #endif
     char *res;
@@ -156,7 +157,6 @@ AdvFlockCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Obj
             errnoval = errno;
         }
 #else
-#if HAVE_LOCKF
         if (ounlock) {
             operation = F_ULOCK;
         }
@@ -187,9 +187,6 @@ AdvFlockCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Obj
             Tcl_SetResult(interp, (void *) "Seek error", TCL_STATIC);
             return TCL_ERROR;
         }
-#else
-#error no available locking implementation
-#endif /* HAVE_LOCKF */
 #endif /* HAVE_FLOCK */
         /* disable the alarm timer */
         alarm(0);
