@@ -33,7 +33,6 @@
 
 package provide portinstall 1.0
 package require portutil 1.0
-package require registry2 2.0
 package require machista 1.0
 
 set org.macports.install [target_new org.macports.install portinstall::install_main]
@@ -55,22 +54,17 @@ set_ui_prefix
 
 proc portinstall::install_start {args} {
     global UI_PREFIX subport version revision portvariants \
-           prefix registry_open registry.path
+           prefix_frozen
     ui_notice "$UI_PREFIX [format [msgcat::mc "Installing %s @%s_%s%s"] $subport $version $revision $portvariants]"
     
     # start gsoc08-privileges
-    if {![file writable $prefix] || ([getuid] == 0 && [geteuid] != 0)} {
+    if {![file writable $prefix_frozen] || ([getuid] == 0 && [geteuid] != 0)} {
         # if install location is not writable, need root privileges to install
         # Also elevate if started as root, since 'file writable' doesn't seem
         # to take euid into account.
         elevateToRoot "install"
     }
     # end gsoc08-privileges
-    
-    if {![info exists registry_open]} {
-        registry::open [file join ${registry.path} registry registry.db]
-        set registry_open yes
-    }
 
     # create any users and groups needed by the port
     handle_add_users
@@ -78,11 +72,10 @@ proc portinstall::install_start {args} {
 
 proc portinstall::create_archive {location archive.type} {
     global workpath destpath portpath subport version revision portvariants \
-           epoch configure.cxx_stdlib portinstall::actual_cxx_stdlib \
-           portinstall::file_is_binary portinstall::cxx_stdlib_overridden \
-           cxx_stdlib PortInfo installPlist \
+           epoch configure.cxx_stdlib cxx_stdlib PortInfo \
            archive.env archive.cmd archive.pre_args archive.args \
-           archive.post_args archive.dir depends_lib depends_run
+           archive.post_args archive.dir depends_lib depends_run \
+           portarchive_hfscompression
     set archive.env {}
     set archive.cmd {}
     set archive.pre_args {}
@@ -91,6 +84,18 @@ proc portinstall::create_archive {location archive.type} {
     set archive.dir ${destpath}
 
     switch -regex -- ${archive.type} {
+        aar {
+            set aa "aa"
+            if {[catch {set aa [findBinary $aa ${portutil::autoconf::aa_path}]} errmsg] == 0} {
+                ui_debug "Using $aa"
+                set archive.cmd "$aa"
+                set archive.pre_args "archive -v"
+                set archive.args "-o [shellescape ${location}] -d ."
+            } else {
+                ui_debug $errmsg
+                return -code error "No '$aa' was found on this system!"
+            }
+        }
         cp(io|gz) {
             set pax "pax"
             if {[catch {set pax [findBinary $pax ${portutil::autoconf::pax_path}]} errmsg] == 0} {
@@ -102,20 +107,20 @@ proc portinstall::create_archive {location archive.type} {
                     if {[catch {set gzip [findBinary $gzip ${portutil::autoconf::gzip_path}]} errmsg] == 0} {
                         ui_debug "Using $gzip"
                         set archive.args {.}
-                        set archive.post_args "| $gzip -c9 > ${location}"
+                        set archive.post_args "| $gzip -c9 > [shellescape ${location}]"
                     } else {
                         ui_debug $errmsg
                         return -code error "No '$gzip' was found on this system!"
                     }
                 } else {
-                    set archive.args "-f ${location} ."
+                    set archive.args "-f [shellescape ${location}] ."
                 }
             } else {
                 ui_debug $errmsg
                 return -code error "No '$pax' was found on this system!"
             }
         }
-        t(ar|bz|lz|xz|gz) {
+        t(ar|bz|lz|xz|gz|mptar) {
             set tar "tar"
             if {[catch {set tar [findBinary $tar ${portutil::autoconf::tar_path}]} errmsg] == 0} {
                 ui_debug "Using $tar"
@@ -149,13 +154,31 @@ proc portinstall::create_archive {location archive.type} {
                     if {[catch {set gzip [findBinary $gzip $hint]} errmsg] == 0} {
                         ui_debug "Using $gzip"
                         set archive.args {- .}
-                        set archive.post_args "| $gzip -c$level > ${location}"
+                        set archive.post_args "| $gzip -c$level > [shellescape ${location}]"
                     } else {
                         ui_debug $errmsg
                         return -code error "No '$gzip' was found on this system!"
                     }
                 } else {
-                    set archive.args "${location} ."
+                    if {${archive.type} eq "tmptar"} {
+                        # Pass through tar for hardlink detection and HFS compression,
+                        # but extract without saving the tar file.
+                        if {${portarchive_hfscompression} && [getuid] == 0 &&
+                            ![catch {binaryInPath bsdtar}] &&
+                            ![catch {exec bsdtar -x --hfsCompression < /dev/null >& /dev/null}]
+                        } then {
+                            set extract_tar bsdtar
+                            set extract_tar_args {-xvp --hfsCompression -f}
+                        } else {
+                            set extract_tar $tar
+                            set extract_tar_args {-xvpf}
+                        }
+                        set archive.args {- .}
+                        set archive.post_args "| $extract_tar -C $location $extract_tar_args -"
+                        file mkdir $location
+                    } else {
+                        set archive.args "[shellescape ${location}] ."
+                    }
                 }
             } else {
                 ui_debug $errmsg
@@ -168,7 +191,7 @@ proc portinstall::create_archive {location archive.type} {
                 ui_debug "Using $xar"
                 set archive.cmd "$xar"
                 set archive.pre_args {-cvf}
-                set archive.args "${location} ."
+                set archive.args "[shellescape ${location}] ."
             } else {
                 ui_debug $errmsg
                 return -code error "No '$xar' was found on this system!"
@@ -180,7 +203,7 @@ proc portinstall::create_archive {location archive.type} {
                 ui_debug "Using $zip"
                 set archive.cmd "$zip"
                 set archive.pre_args {-ry9}
-                set archive.args "${location} ."
+                set archive.args "[shellescape ${location}] ."
             } else {
                 ui_debug $errmsg
                 return -code error "No '$zip' was found on this system!"
@@ -242,15 +265,15 @@ proc portinstall::create_archive {location archive.type} {
     puts $fd "@portversion ${version}"
     puts $fd "@portrevision ${revision}"
     puts $fd "@archs [get_canonical_archs]"
-    array set ourvariations $PortInfo(active_variants)
-    set vlist [lsort -ascii [array names ourvariations]]
+    set ourvariations $PortInfo(active_variants)
+    set vlist [lsort -ascii [dict keys $ourvariations]]
     foreach v $vlist {
-        if {$ourvariations($v) eq "+"} {
+        if {[dict get $ourvariations $v] eq "+"} {
             puts $fd "@portvariant +${v}"
         }
     }
 
-    foreach key "depends_lib depends_run" {
+    foreach key [list depends_lib depends_run] {
          if {[info exists $key]} {
              foreach depspec [set $key] {
                  set depname [lindex [split $depspec :] end]
@@ -258,19 +281,20 @@ proc portinstall::create_archive {location archive.type} {
                  if {[llength $dep] < 2} {
                      ui_debug "Dependency $depname not found"
                  } else {
-                     array set portinfo [lindex $dep 1]
-                     set depver $portinfo(version)
-                     set deprev $portinfo(revision)
-                     puts $fd "@pkgdep $portinfo(name)-${depver}_${deprev}"
+                     lassign $dep depname dep_portinfo
+                     set depver [dict get $dep_portinfo version]
+                     set deprev [dict get $dep_portinfo revision]
+                     puts $fd "@pkgdep ${depname}-${depver}_${deprev}"
                  }
              }
          }
     }
 
     set have_fileIsBinary [expr {[option os.platform] eq "darwin"}]
-    set binary_files {}
+    set binary_files [list]
+    variable file_is_binary [dict create]
     # also save the contents for our own use later
-    set installPlist {}
+    variable installPlist [list]
     set destpathLen [string length $destpath]
     fs-traverse -depth fullpath [list $destpath] {
         if {[file type $fullpath] eq "directory"} {
@@ -293,7 +317,7 @@ proc portinstall::create_archive {location archive.type} {
                         lappend binary_files $fullpath
                     }
                     puts $fd "@comment binary:$is_binary"
-                    set portinstall::file_is_binary($abspath) $is_binary
+                    dict set file_is_binary $abspath $is_binary
                 }
             }
         } else {
@@ -304,20 +328,20 @@ proc portinstall::create_archive {location archive.type} {
         puts $fd "@ignore"
         puts $fd "$relpath"
     }
-    set portinstall::actual_cxx_stdlib [get_actual_cxx_stdlib $binary_files]
-    puts $fd "@cxx_stdlib ${portinstall::actual_cxx_stdlib}"
-    if {${portinstall::actual_cxx_stdlib} ne "none"} {
-        set portinstall::cxx_stdlib_overridden [expr {${configure.cxx_stdlib} ne $cxx_stdlib}]
+    variable actual_cxx_stdlib [get_actual_cxx_stdlib $binary_files]
+    puts $fd "@cxx_stdlib ${actual_cxx_stdlib}"
+    if {${actual_cxx_stdlib} ne "none"} {
+        variable cxx_stdlib_overridden [expr {${configure.cxx_stdlib} ne $cxx_stdlib}]
     } else {
-        set portinstall::cxx_stdlib_overridden 0
+        variable cxx_stdlib_overridden 0
     }
-    puts $fd "@cxx_stdlib_overridden ${portinstall::cxx_stdlib_overridden}"
+    puts $fd "@cxx_stdlib_overridden ${cxx_stdlib_overridden}"
     close $fd
 
     # Now create the archive
     ui_debug "Creating [file tail $location]"
     command_exec archive
-    ui_debug "Archive [file tail $location] packaged"
+    ui_debug "Port image [file tail $location] created"
 
     # Cleanup all control files when finished
     set control_files [glob -nocomplain -types f [file join $destpath +*]]
@@ -327,15 +351,14 @@ proc portinstall::create_archive {location archive.type} {
     }
 }
 
-proc portinstall::extract_contents {location type} {
-    return [extract_archive_metadata $location $type contents]
-}
-
 proc portinstall::install_main {args} {
     global subport version portpath depends_run revision user_options \
     portvariants requested_variants depends_lib PortInfo epoch \
-    os.platform os.major portarchivetype installPlist registry.path porturl \
-    portinstall::file_is_binary portinstall::actual_cxx_stdlib portinstall::cxx_stdlib_overridden
+    portarchivetype portimage_mode
+    variable file_is_binary
+    variable actual_cxx_stdlib
+    variable cxx_stdlib_overridden
+    variable installPlist
 
     set oldpwd [pwd]
     if {$oldpwd eq ""} {
@@ -348,24 +371,32 @@ proc portinstall::install_main {args} {
         set install_dir [file dirname $location]
         file mkdir $install_dir
         file rename -force $archive_path $install_dir
+        # Clean up statefile so the state is consistent now that the
+        # archive is moved, in case anything fails from here on.
+        delete [file join [option workpath] .macports.${subport}.state]
         set location [file join $install_dir [file tail $archive_path]]
         set current_archive_type [string range [file extension $location] 1 end]
-        set contents [extract_contents $location $current_archive_type]
-        set installPlist [lindex $contents 0]
-        array set portinstall::file_is_binary [lindex $contents 1]
-        set cxxinfo [extract_archive_metadata $location $current_archive_type cxx_info]
-        set portinstall::actual_cxx_stdlib [lindex $cxxinfo 0]
-        set portinstall::cxx_stdlib_overridden [lindex $cxxinfo 1]
+        set archive_metadata [extract_archive_metadata $location $current_archive_type {contents cxx_info}]
+        lassign [dict get $archive_metadata contents] installPlist file_is_binary
+        lassign [dict get $archive_metadata cxx_info] actual_cxx_stdlib cxx_stdlib_overridden
     } else {
-        # throws an error if an unsupported value has been configured
-        archiveTypeIsSupported $portarchivetype
+        if {$portimage_mode eq "directory"} {
+            # Special value to avoid writing archive out to disk, since
+            # only the extracted dir should be kept.
+            set archivetype tmptar
+            set location [file rootname $location]
+        } else {
+            # throws an error if an unsupported value has been configured
+            archiveTypeIsSupported $portarchivetype
+            set archivetype $portarchivetype
+        }
         # create archive from the destroot
-        create_archive $location $portarchivetype
+        create_archive $location $archivetype
     }
 
     # can't do this inside the write transaction due to deadlock issues with _get_dep_port
     set dep_portnames [list]
-    foreach deplist {depends_lib depends_run} {
+    foreach deplist [list depends_lib depends_run] {
         if {[info exists $deplist]} {
             foreach dep [set $deplist] {
                 set dep_portname [_get_dep_port $dep]
@@ -376,88 +407,62 @@ proc portinstall::install_main {args} {
         }
     }
 
-    registry::write {
-
-        set regref [registry::entry create $subport $version $revision $portvariants $epoch]
-
-        if {[info exists user_options(ports_requested)]} {
-            $regref requested $user_options(ports_requested)
-        } else {
-            $regref requested 0
-        }
-        $regref os_platform ${os.platform}
-        $regref os_major ${os.major}
-        $regref archs [get_canonical_archs]
-        if {${portinstall::actual_cxx_stdlib} ne ""} {
-            $regref cxx_stdlib ${portinstall::actual_cxx_stdlib}
-            $regref cxx_stdlib_overridden ${portinstall::cxx_stdlib_overridden}
-        } else {
-            # no info in the archive
-            global configure.cxx_stdlib cxx_stdlib
-            $regref cxx_stdlib_overridden [expr {${configure.cxx_stdlib} ne $cxx_stdlib}]
-        }
-        # Trick to have a portable GMT-POSIX epoch-based time.
-        $regref date [expr {[clock scan now -gmt true] - [clock scan "1970-1-1 00:00:00" -gmt true]}]
-        if {[info exists requested_variants]} {
-            $regref requested_variants $requested_variants
-        }
-
-        foreach dep_portname $dep_portnames {
-            $regref depends $dep_portname
-        }
-
-        $regref installtype image
-        $regref state imaged
-        $regref location $location
-
-        if {[info exists installPlist]} {
-            # register files
-            $regref map $installPlist
-            foreach f [array names portinstall::file_is_binary] {
-                set fileref [registry::file open [$regref id] $f]
-                $fileref binary $portinstall::file_is_binary($f)
-                registry::file close $fileref
-            }
-        }
-
-        # store portfile
-        set portfile_path [file join $portpath Portfile]
-        set portfile_sha256 [sha256 file $portfile_path]
-        set portfile_size [file size $portfile_path]
-        set portfile_reg_dir [file join ${registry.path} registry portfiles ${subport}-${version}_${revision} ${portfile_sha256}-${portfile_size}]
-        file mkdir $portfile_reg_dir
-        set portfile_reg_path ${portfile_reg_dir}/Portfile
-        if {![file isfile $portfile_reg_path] || [file size $portfile_reg_path] != $portfile_size || [sha256 file $portfile_reg_path] ne $portfile_sha256} {
-            file copy -force $portfile_path $portfile_reg_dir
-            file attributes $portfile_reg_path -permissions 0644
-        }
-        $regref portfile ${portfile_sha256}-${portfile_size}
-
-        # store portgroups
-        if {[info exists PortInfo(portgroups)]} {
-            foreach pg $PortInfo(portgroups) {
-                set pgname [lindex $pg 0]
-                set pgversion [lindex $pg 1]
-                set groupFile [lindex $pg 2]
-                if {[file isfile $groupFile]} {
-                    set pgsha256 [sha256 file $groupFile]
-                    set pgsize [file size $groupFile]
-                    set pg_reg_dir [file join ${registry.path} registry portgroups ${pgsha256}-${pgsize}]
-                    set pg_reg_path ${pg_reg_dir}/${pgname}-${pgversion}.tcl
-                    if {![file isfile $pg_reg_path] || [file size $pg_reg_path] != $pgsize || [sha256 file $pg_reg_path] ne $pgsha256} {
-                        file mkdir $pg_reg_dir
-                        file copy -force $groupFile $pg_reg_dir
-                    }
-                    file attributes $pg_reg_path -permissions 0644
-                    $regref addgroup $pgname $pgversion $pgsha256 $pgsize
-                } else {
-                    ui_debug "install_main: no portgroup ${pgname}-${pgversion}.tcl found"
-                }
-            }
-        }
+    set regref [dict create]
+    dict set regref name $subport
+    dict set regref version $version
+    dict set regref revision $revision
+    dict set regref variants $portvariants
+    dict set regref epoch $epoch
+    if {[info exists user_options(ports_requested)]} {
+        dict set regref requested $user_options(ports_requested)
+    } else {
+        dict set regref requested 0
+    }
+    lassign [_get_compatible_platform] os_platform os_major
+    dict set regref os_platform $os_platform
+    dict set regref os_major $os_major
+    dict set regref archs [get_canonical_archs]
+    if {${actual_cxx_stdlib} ne ""} {
+        dict set regref cxx_stdlib ${actual_cxx_stdlib}
+        dict set regref cxx_stdlib_overridden ${cxx_stdlib_overridden}
+    } else {
+        # no info in the archive
+        global configure.cxx_stdlib cxx_stdlib
+        dict set regref cxx_stdlib_overridden [expr {${configure.cxx_stdlib} ne $cxx_stdlib}]
+    }
+    # Trick to have a portable GMT-POSIX epoch-based time.
+    dict set regref date [expr {[clock scan now -gmt true] - [clock scan "1970-1-1 00:00:00" -gmt true]}]
+    if {[info exists requested_variants]} {
+        dict set regref requested_variants $requested_variants
     }
 
-    #registry::entry close $regref
+    dict set regref depends $dep_portnames
+
+    dict set regref location $location
+
+    if {[info exists installPlist]} {
+        dict set regref files $installPlist
+        dict set regref binary $file_is_binary
+    }
+
+    # portfile info
+    dict set regref portfile_path [file join $portpath Portfile]
+
+    # portgroup info
+    if {[info exists PortInfo(portgroups)]} {
+        set regref_pgs [list]
+        foreach pg $PortInfo(portgroups) {
+            lassign $pg pgname pgversion groupFile
+            if {[file isfile $groupFile]} {
+                lappend regref_pgs $pgname $pgversion $groupFile
+            } else {
+                ui_debug "install_main: no portgroup ${pgname}-${pgversion}.tcl found"
+            }
+        }
+        dict set regref portgroups $regref_pgs
+    }
+
+    registry_install $regref
 
     _cd $oldpwd
     return 0
