@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 #
-# Copyright (c) 2007 - 2016 The MacPorts Project
+# Copyright (c) 2007 - 2018, 2020 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,7 @@
 
 package provide portlint 1.0
 package require portutil 1.0
+package require portchecksum 1.0
 
 set org.macports.lint [target_new org.macports.lint portlint::lint_main]
 target_runtype ${org.macports.lint} always
@@ -47,6 +48,7 @@ set lint_portsystem \
     "1.0"
 
 set lint_platforms [list \
+    "any" \
     "darwin" \
     "freebsd" \
     "linux" \
@@ -115,6 +117,164 @@ proc portlint::seems_utf8 {str} {
     return true
 }
 
+# lint_checksum_types
+#
+# Given a list of checksum types, return a list of strings which are warnings
+# about deprecated checksum types, or missing recommended types.
+#
+# Returns an empty list if no issues are found.
+proc portlint::lint_checksum_type_list {types} {
+    set issues [list]
+    set using_secure false
+
+    foreach preferred $portchecksum::secure_checksum_types {
+        if {$preferred ni $types} {
+            lappend issues "missing recommended checksum type: $preferred"
+        } else {
+            set using_secure true
+        }
+    }
+    global ports_lint_nitpick
+    if {[tbool ports_lint_nitpick] && "size" ni $types} {
+        lappend issues "missing recommended checksum type: size"
+    }
+
+    if {!$using_secure} {
+        foreach type $types {
+            lappend issues "checksum type is insecure on its own: $type"
+        }
+    }
+
+    return $issues
+}
+
+# lint_checksum
+#
+# Checks a given Portfile checksum string.  Returns a list of lists.
+# The first member list is a list of error strings.
+# The second member list is a list of warning strings.
+#
+# Returns a list containing two empty lists if no issues are found.
+proc portlint::lint_checksum {checksum_string} {
+    set errors [list]
+    set warnings [list]
+
+    set is_error false
+    set ctr_start 0
+
+    set filename ""
+    set pfx ""
+    set has_filenames false
+
+    set types [list]
+
+    # List of all tokens in the checksum string
+    set checksum_tokens [regexp -all -inline {\S+} $checksum_string]
+
+    if {[lindex $checksum_tokens 0] eq "checksum"} {
+        incr ctr_start
+    }
+
+    for {set ctr $ctr_start} \
+        {($ctr < [llength $checksum_tokens]) && !$is_error} \
+        {} {
+
+        set current [lindex $checksum_tokens $ctr]
+
+        if {$current in $portchecksum::checksum_types} {
+            set c_type  $current
+            set c_value [lindex $checksum_tokens $ctr+1]
+
+            switch [portchecksum::verify_checksum_format $c_type $c_value] {
+                1 {
+                    # checksum type recognized, and checksum looks good
+                    incr ctr 2
+                    lappend types $c_type
+                }
+
+                0 {
+                    # checksum type recognized, but checksum looks bad
+                    lappend errors "${pfx}checksum type $c_type, but\
+                                    checksum is invalid: $c_value"
+                    incr ctr 2
+                }
+
+                -1 {
+                    # checksum type not recognized
+                    lappend errors "${pfx}invalid checksum type: $c_type\
+                                    $c_value"
+                    set is_error true
+                    continue
+                }
+            }
+
+        } elseif {($ctr > $ctr_start) && !$has_filenames} {
+            lappend errors "invalid checksum field: $current"
+            set is_error true
+            continue
+        } else {
+            if {$ctr == $ctr_start} {
+                set has_filenames true
+            } elseif {($ctr == ([llength $checksum_tokens] - 1)) || \
+                         ([portchecksum::verify_checksum_format \
+                            [lindex $checksum_tokens $ctr-2] \
+                            [lindex $checksum_tokens $ctr-1] \
+                          ] != 1)} {
+                lappend errors "invalid checksum field: $current"
+                set is_error true
+                continue
+            }
+
+            if {[llength $types] > 0} {
+                set types_lint [portlint::lint_checksum_type_list $types]
+                foreach lint_issue $types_lint {
+                    lappend warnings "${pfx}${lint_issue}"
+                }
+            }
+
+            set filename $current
+            set pfx "$filename - "
+            set types [list]
+            incr ctr
+        }
+    }
+
+    if {[llength $types] > 0} {
+        set types_lint [portlint::lint_checksum_type_list $types]
+        foreach lint_issue $types_lint {
+            lappend warnings "${pfx}${lint_issue}"
+        }
+    }
+
+    return [list $errors $warnings]
+}
+
+# lint_platforms
+#
+# Checks a given Portfile platforms string.  Returns a list of lists.
+# The first member list is a list of error strings.
+# The second member list is a list of warning strings.
+#
+# Returns a list containing two empty lists if no issues are found.
+proc portlint::lint_platforms {platforms} {
+    set errors [list]
+    set warnings [list]
+
+    global lint_platforms
+
+    foreach platform $platforms {
+        set platname [lindex $platform 0]
+        if {$platname ni $lint_platforms} {
+            lappend errors "Unknown platform: $platname"
+        }
+    }
+
+    if {$platforms eq "darwin"} {
+        lappend warnings "Unnecessary platforms line as darwin is the default"
+    }
+
+    return [list $errors $warnings]
+}
 
 proc portlint::lint_start {args} {
     global UI_PREFIX subport
@@ -122,7 +282,8 @@ proc portlint::lint_start {args} {
 }
 
 proc portlint::lint_main {args} {
-    global UI_PREFIX name portpath porturl ports_lint_nitpick
+    global UI_PREFIX portpath ports_lint_nitpick lint_required lint_optional
+    global {*}$lint_required {*}$lint_optional
     set portfile ${portpath}/Portfile
     set portdirs [split ${portpath} /]
     set last [llength $portdirs]
@@ -148,9 +309,11 @@ proc portlint::lint_main {args} {
     set require_after ""
     set seen_portsystem false
     set seen_portgroup false
+    set seen_platforms false
     set in_description false
+    set prohibit_tabs false
 
-    array set portgroups {}
+    set portgroups [dict create]
 
     set local_variants [list]
 
@@ -162,13 +325,11 @@ proc portlint::lint_main {args} {
     while {1} {
         set line [gets $f]
         if {[eof $f]} {
-            if {$nitpick} {
-                seek $f -1 end
-                set last [read $f 1]
-                if {"\n" ne $last} {
-                    ui_warn "Line $lineno has missing newline (at end of file)"
-                    incr warnings
-                }
+            seek $f -1 end
+            set last [read $f 1]
+            if {"\n" ne $last} {
+                ui_warn "Line $lineno has missing newline (at end of file)"
+                incr warnings
             }
             close $f
             break
@@ -181,7 +342,7 @@ proc portlint::lint_main {args} {
         }
 
         if {($require_after eq "PortSystem" || $require_after eq "PortGroup") && \
-            [string match "PortGroup*" $line]} {
+            [regexp {^\s*PortGroup\s} $line]} {
             set require_blank false
         }
 
@@ -200,6 +361,16 @@ proc portlint::lint_main {args} {
         if {($lineno == $topline_number) && [string match "*-\*- *" $line]} {
             ui_info "OK: Line $lineno has emacs/vim Mode"
             incr topline_number
+            set require_blank true
+            set require_after "modeline"
+            if {[regexp {\sindent-tabs-mode: nil[;\s]|[:\s](?:et|expandtab)(?:[:\s]|$)} $line]} {
+                set prohibit_tabs true
+            }
+        }
+
+        if {$prohibit_tabs && [string match "*\t*" $line]} {
+            ui_warn "Line $lineno contains tab but modeline says tabs should be expanded"
+            incr warnings
         }
 
         if {[string match "*\$Id*\$" $line]} {
@@ -213,12 +384,12 @@ proc portlint::lint_main {args} {
             continue
         }
 
-        if {[string match "PortSystem*" $line]} {
+        if {[regexp {^\s*PortSystem\s} $line]} {
             if {$seen_portsystem} {
                 ui_error "Line $lineno repeats PortSystem declaration"
                 incr errors
             }
-            regexp {PortSystem\s+([0-9.]+)} $line -> portsystem
+            regexp {^\s*PortSystem\s+([0-9.]+)\s*$} $line -> portsystem
             if {![info exists portsystem]} {
                 ui_error "Line $lineno has unrecognized PortSystem"
                 incr errors
@@ -227,29 +398,32 @@ proc portlint::lint_main {args} {
             set require_blank true
             set require_after "PortSystem"
         }
-        if {[string match "PortGroup*" $line]} {
-            regexp {PortGroup\s+([A-Za-z0-9_]+)\s+([0-9.]+)} $line -> portgroup portgroupversion
+        if {[regexp {^\s*PortGroup\s} $line]} {
+            regexp {^\s*PortGroup\s+([A-Za-z0-9_]+)\s+([0-9.]+)\s*$} $line -> portgroup portgroupversion
             if {![info exists portgroup]} {
                 ui_error "Line $lineno has unrecognized PortGroup"
                 incr errors
             } else {
-                if {[info exists portgroups($portgroup)]} {
+                if {[dict exists $portgroups $portgroup]} {
                     ui_error "Line $lineno repeats inclusion of PortGroup $portgroup"
                     incr errors
                 } else {
-                    set portgroups($portgroup) $portgroupversion
+                    dict set portgroups $portgroup $portgroupversion
                 }
             }
             set seen_portgroup true
             set require_blank true
             set require_after "PortGroup"
         }
+        if {[regexp {^\s*platforms\s} $line]} {
+            set seen_platforms true
+        }
 
         # TODO: check for repeated variable definitions
         # TODO: check the definition order of variables
         # TODO: check length of description against max
 
-        if {[string match "long_description*" $line]} {
+        if {[regexp {^\s*long_description\s} $line]} {
             set in_description true
         }
         if {$in_description && ([string range $line end end] ne "\\")} {
@@ -260,32 +434,40 @@ proc portlint::lint_main {args} {
             set require_blank false
         }
 
-        if {[string match "variant*" $line]} {
-            regexp {variant\s+(\w+)} $line -> variantname
+        if {[regexp {^\s*variant\s} $line]} {
+            regexp {^\s*variant\s+(\w+)} $line -> variantname
             if {[info exists variantname]} {
                 lappend local_variants $variantname
             }
         }
         
-        if {[string match "platform\[ \t\]*" $line]} {
-            regexp {platform\s+(?:\w+\s+(?:\w+\s+)?)?(\w+)} $line -> platform_arch
-            if {$platform_arch eq "ppc"} {
-                ui_error "Arch 'ppc' in platform on line $lineno should be 'powerpc'"
-                incr errors
+        if {[regexp {^\s*platform\s} $line]} {
+            regexp {^\s*platform\s+(?:\w+\s+(?:\w+\s+)?)?(\w+)} $line -> platform_arch
+            foreach {bad_platform_arch replacement_platform_arch} {
+                arm64 arm
+                intel i386
+                ppc powerpc
+                ppc64 powerpc
+                x86_64 i386
+            } {
+                if {$platform_arch eq $bad_platform_arch} {
+                    ui_error "Arch '$bad_platform_arch' in platform on line $lineno should be '$replacement_platform_arch'"
+                    incr errors
+                }
             }
         }
 
-        if {[string match "*adduser*" $line]} {
+        if {[regexp {^\s*adduser\s} $line]} {
             ui_warn "Line $lineno calling adduser directly; consider setting add_users instead"
             incr warnings
         }
 
-        if {[regexp {(^|\s)configure\s+\{\s*\}} $line]} {
+        if {[regexp {^\s*configure\s+\{\s*\}} $line]} {
             ui_warn "Line $lineno should say \"use_configure no\" instead of declaring an empty configure phase"
             incr warnings
         }
 
-        if {[regexp {compiler\.blacklist(?:-[a-z]+)?\s.*(["{]\S+(?:\s+\S+){2,}["}])} $line -> blacklist] && ![info exists portgroups(compiler_blacklist_versions)]} {
+        if {[regexp {^\s*compiler\.blacklist(?:-[a-z]+)?\s.*(["{]\S+(?:\s+\S+){2,}["}])} $line -> blacklist] && ![dict exists $portgroups compiler_blacklist_versions]} {
             ui_error "Line $lineno uses compiler.blacklist entry $blacklist which requires the compiler_blacklist_versions portgroup which has not been included"
             incr errors
         }
@@ -301,16 +483,14 @@ proc portlint::lint_main {args} {
         # Check for hardcoded version numbers
         if {$nitpick} {
             # Support for skipping checksums lines
-            if {[regexp {^checksums} $line]} {
+            if {[regexp {^\s*checksums\s} $line]} {
                 # We enter a series of one or more lines containing checksums
                 set hashline true
             }
     
             if {!$hashline
-                    && ![regexp {^\s*PortSystem|^\s*PortGroup|^\s*version} $line]
-                    && ![regexp {^\s*[A-Za-z0-9_]+\.setup} $line]
-                    && ![regexp {^\s*license} $line]
-                    && [string first [option version] $line] != -1} {
+                    && ![regexp {^\s*(?:PortSystem|PortGroup|version|python\.versions|(?:perl5|php|ruby)\.branch(?:es)?|license|[A-Za-z0-9_]+\.setup)\s} $line]
+                    && [string first $version $line] != -1} {
                 ui_warn "Line $lineno seems to hardcode the version number, consider using \${version} instead"
                 incr warnings
             }
@@ -327,12 +507,16 @@ proc portlint::lint_main {args} {
         if {!$hashline
                 && $name ne "MacPorts"
                 && [string match "*/opt/local*" $line]
-                && ![regexp {^\s*reinplace} $line]
-                && ![regexp {^\s*system.*\Wsed\W} $line]} {
+                && ![regexp {^\s*(?:reinplace\s|system.*\Wsed\W)} $line]} {
             ui_error "Line $lineno hardcodes /opt/local, use \${prefix} instead"
             incr errors
         }
 
+        if {[regexp {\$\{?macosx_version} $line]} {
+            ui_warn "Line $lineno using macosx_version; switch to macos_version or macos_version_major"
+            incr warnings
+        }
+
         ### TODO: more checks to Portfile syntax
 
         incr lineno
@@ -340,12 +524,11 @@ proc portlint::lint_main {args} {
 
     ###################################################################
 
-    global os.platform os.arch os.version version revision epoch \
-           description long_description platforms categories all_variants \
-           maintainers license homepage master_sites checksums patchfiles \
-           depends_fetch depends_extract depends_lib depends_build depends_run \
-           depends_test distfiles fetch.type lint_portsystem lint_platforms \
-           lint_required lint_optional replaced_by conflicts
+    global porturl portutil::all_variants patchfiles \
+           depends_fetch depends_extract depends_patch \
+           depends_lib depends_build depends_run \
+           depends_test distfiles fetch.type lint_portsystem \
+           replaced_by conflicts
     set portarch [get_canonical_archs]
 
     if {!$seen_portsystem} {
@@ -360,7 +543,7 @@ proc portlint::lint_main {args} {
 
     if {$seen_portgroup} {
         # Using a PortGroup is optional
-        foreach {portgroup portgroupversion} [array get portgroups] {
+        dict for {portgroup portgroupversion} $portgroups {
             if {![file exists [getportresourcepath $porturl "port1.0/group/${portgroup}-${portgroupversion}.tcl"]]} {
                 ui_error "Unknown PortGroup: $portgroup-$portgroupversion"
                 incr errors
@@ -371,7 +554,6 @@ proc portlint::lint_main {args} {
     }
 
     foreach req_var $lint_required {
-
         if {$req_var eq "master_sites"} {
             if {${fetch.type} ne "standard"} {
                 ui_info "OK: $req_var not required for fetch.type ${fetch.type}"
@@ -405,14 +587,22 @@ proc portlint::lint_main {args} {
         }
     }
 
-    if {[info exists platforms]} {
-        foreach platform $platforms {
-            if {$platform ni $lint_platforms} {
-                ui_error "Unknown platform: $platform"
-                incr errors
-            } else {
-                ui_info "OK: Found platform: $platform"
-            }
+    if {[info exists version] && [string match *+* $version]} {
+        ui_warn "Version '$version' contains '+' character, which makes it difficult to specify on the command line due to conflict with variant syntax."
+        incr warnings
+    }
+
+    if {$seen_platforms && [info exists platforms]} {
+        set platforms_lint [portlint::lint_platforms $platforms]
+
+        foreach err [lindex $platforms_lint 0] {
+            ui_error $err
+            incr errors
+        }
+
+        foreach warning [lindex $platforms_lint 1] {
+            ui_warn $warning
+            incr warnings
         }
     }
 
@@ -437,27 +627,31 @@ proc portlint::lint_main {args} {
             set name_ok true
             set desc_ok true
 
-            if {![regexp {^[A-Za-z0-9_]+$} $variantname]} {
-                ui_error "Variant name $variantname is not valid; use \[A-Za-z0-9_\]+ only"
+            if {![regexp {^[A-Za-z0-9_.]+$} $variantname]} {
+                ui_error "Variant name $variantname is not valid; use \[A-Za-z0-9_.\]+ only"
                 incr errors
                 set name_ok false
             }
 
+            global portresourcepath
+            if {![info exists portresourcepath]} {
+                global porturl
+                set portresourcepath [getportresourcepath $porturl]
+            }
+
             if {![info exists variantdesc] || $variantdesc eq ""} {
                 # don't warn about missing descriptions for global variants
                 if {$variantname in $local_variants &&
-                    [variant_desc $porturl $variantname] eq ""} {
+                    [get_variant_description $variantname $portresourcepath] eq ""} {
                     ui_warn "Variant $variantname does not have a description"
                     incr warnings
                     set desc_ok false
                 } elseif {$variantdesc eq ""} {
                     set variantdesc "(pre-defined variant)"
                 }
-            } else {
-                if {[variant_desc $porturl $variantname] ne ""} {
-                    ui_warn "Variant $variantname overrides global description"
-                    incr warnings
-                }
+            } elseif {[get_variant_description $variantname $portresourcepath] ne ""} {
+                ui_warn "Variant $variantname overrides global description"
+                incr warnings
             }
 
             # Check if conflicting variants actually exist
@@ -486,13 +680,16 @@ proc portlint::lint_main {args} {
         incr variantnumber
     }
 
-    set all_depends {}
+    set all_depends [list]
     if {[info exists depends_fetch]} {
         lappend all_depends {*}$depends_fetch
     }
     if {[info exists depends_extract]} {
         lappend all_depends {*}$depends_extract
     }
+    if {[info exists depends_patch]} {
+        lappend all_depends {*}$depends_patch
+    }
     if {[info exists depends_lib]} {
         lappend all_depends {*}$depends_lib
     }
@@ -520,16 +717,16 @@ proc portlint::lint_main {args} {
     }
 
     # Check for multiple dependencies
-    foreach deptype {depends_extract depends_lib depends_build depends_run depends_test} {
+    foreach deptype {depends_extract depends_patch depends_lib depends_build depends_run depends_test} {
         if {[info exists $deptype]} {
-            array set depwarned {}
+            set depwarned [dict create]
             foreach depspec [set $deptype] {
-                if {![info exists depwarned($depspec)]
+                if {![dict exists $depwarned $depspec]
                         && [llength [lsearch -exact -all [set $deptype] $depspec]] > 1} {
                     ui_warn "Dependency $depspec specified multiple times in $deptype"
                     incr warnings
                     # Report each depspec only once
-                    set depwarned($depspec) yes
+                    dict set depwarned $depspec yes
                 }
             }
         }
@@ -552,6 +749,20 @@ proc portlint::lint_main {args} {
         }
     }
 
+    if {[info exists checksums]} {
+        set checksum_lint [portlint::lint_checksum $checksums]
+
+        foreach err [lindex $checksum_lint 0] {
+            ui_error $err
+            incr errors
+        }
+
+        foreach warning [lindex $checksum_lint 1] {
+            ui_warn $warning
+            incr warnings
+        }
+    }
+
     if {[info exists conflicts]} {
         foreach cport $conflicts {
             if {[regexp {[^[:alnum:]_.-]} $cport]} {
@@ -572,12 +783,12 @@ proc portlint::lint_main {args} {
         }
     }
 
-    if {[regexp "^(.+)nomaintainer(@macports.org)?(.+)$" $maintainers] } {
+    if {[regexp "^(.+)nomaintainer(@macports\.org)?(.+)$" $maintainers] } {
         ui_error "Using nomaintainer together with other maintainer"
         incr errors
     }
 
-    if {[regexp "^openmaintainer(@macports.org)?$" $maintainers] } {
+    if {[regexp "^openmaintainer(@macports\.org)?$" $maintainers] } {
         ui_error "Using openmaintainer without any other maintainer"
         incr errors
     }
@@ -588,10 +799,10 @@ proc portlint::lint_main {args} {
                     $addr eq "openmaintainer@macports.org"} {
                 ui_warn "Using full email address for no/open maintainer"
                 incr warnings
-            } elseif {[regexp "^(.+)@macports.org$" $addr -> localpart]} {
+            } elseif {[regexp "^(.+)@macports\.org$" $addr -> localpart]} {
                 ui_warn "Maintainer email address for $localpart includes @macports.org"
                 incr warnings
-            } elseif {$addr eq "darwinports@opendarwin.org"} {
+            } elseif {$addr eq "darwinports@opendarwin\.org"} {
                 ui_warn "Using legacy email address for no/open maintainer"
                 incr warnings
             } elseif {[regexp "^(.+)@(.+)$" $addr -> localpart domain]} {
@@ -671,8 +882,10 @@ proc portlint::lint_main {args} {
 
     if {$nitpick && [info exists patchfiles]} {
         foreach patchfile $patchfiles {
-            if {![string match "patch-*.diff" $patchfile] && [file exists "$portpath/files/$patchfile"]} {
-                ui_warn "Patchfile $patchfile does not follow the source patch naming policy \"patch-*.diff\""
+            if {!([string match "*.diff" $patchfile] ||
+                  [string match "*.patch" $patchfile]) &&
+                 [file exists "$portpath/files/$patchfile"]} {
+                ui_warn "Patchfile $patchfile does not follow the source patch naming policy \"*.diff\" or \"*.patch\""
                 incr warnings
             }
         }
